@@ -23,8 +23,18 @@
 #define RSYNC_NAME "rsync"
 #define BACKUP_SUFFIX "~"
 
+#define FILE_VALID 1
+#define SAME_MODE (1<<1)
+#define SAME_DEV (1<<2)
+#define SAME_UID (1<<3)
+#define SAME_GID (1<<4)
+#define SAME_DIR (1<<5)
+#define SAME_NAME SAME_DIR
+#define LONG_NAME (1<<6)
+#define SAME_TIME (1<<7)
+
 /* update this if you make incompatible changes */
-#define PROTOCOL_VERSION 10
+#define PROTOCOL_VERSION 11
 #define MIN_PROTOCOL_VERSION 10
 
 /* block size to write files in */
